require 'json'
require 'active_support/core_ext/hash'
require 'rest-client'
require 'yaml'
require_relative 'macro'
require_relative 'argbucket'

DEFAULT_CONFIG = '/tmp/apidragonconf.yaml'
PLUGINS = '/usr/local/apidragonplugins/'
LOG = '/tmp/apidragon.log'

class Api < ArgBucket
  # Uses the configuration data in config.yaml to run a sequence of api calls.
  # All variables are dumped into a variable bucket (@arg_bucket)
  # so that they can be used as parameters for function calls.

  def initialize(macro_name, config, username, password)
    @config_file = config
    Dir.mkdir PLUGINS unless Dir.exist? PLUGINS
    @arg_bucket = {}
    set 'username', username unless username.nil?
    set 'password', password unless password.nil?
    @config = load_config @config_file
    import @config['vars']
    @macro = @config['macros'][macro_name]
    if @macro.nil? then fail "Command: '#{macro_name}' is not defined." end
  end

  def load_config(config)
    if config.nil? then config = DEFAULT_CONFIG end
    YAML.load_file config
  end

  def go
    @macro.each_pair do |_key, value|
      call = Call.new(value, @arg_bucket)
      @arg_bucket = call.run
      unless value['record'].nil?
        value['record'].each do |var|
          if value['record'].include?(var) then add_config_var var, get(var) end
        end
      end
    end
  end

  def import(args)
    args.each_pair do |key, value|
      set key, value
    end
  end

  def add_config_var(key,value)
    @config['vars'][key] = value
    file = File.open @config_file, 'w'
    file.write(@config.to_yaml.gsub("\n-", "\n\n-"))
    file.close
  end
end
